/***********************************************
 * Example demonstrating SPI transfers with EasyDMA 
 * triggered by TIMER0 and PPI channels. 
 * MADE FOR SDK 12.2.0
 **********************************************/



#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "nrf_drv_timer.h"
#include "bsp.h"
#include "app_error.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_spi.h"
#include "nrf_log.h"
#include "nrf_delay.h"

#define SPI_MISO_PIN    28
#define SPI_MOSI_PIN    4 
#define SPI_SCL_PIN     3 
#define SPI_CS_PIN      29

// Number of bytes per transfer
#define BYTES_PR_XFER   6
// Number of transfers to complete
#define NUMBER_OF_XFERS 20

typedef struct ArrayList
{
    uint8_t buffer[BYTES_PR_XFER];
} ArrayList_type;

static ArrayList_type p_rx_data[NUMBER_OF_XFERS] = {0};
 
// Data for my test slave
uint8_t p_tx_data = 0x67 | 0x80;

static const nrf_drv_spi_t spi = NRF_DRV_SPI_INSTANCE(0);

// TIMER0 Used to start SPI transfers at regular intervals
const nrf_drv_timer_t timer_timer = NRF_DRV_TIMER_INSTANCE(0);
// TIMER1 Used in Counter mode to count number of completed transfers and stop the SPI after NUMBER_OF_XFERS number of transfers
const nrf_drv_timer_t timer_counter = NRF_DRV_TIMER_INSTANCE(1);


// Task and event addresses
uint32_t start_spi_task_addr;
uint32_t spi_end_evt_addr;
uint32_t timer0_timeout_cc0_evt_addr;
uint32_t counter1_count_task_addr;
uint32_t counter1_cc0_evt_addr;


void spi_event_handler(nrf_drv_spi_evt_t const * p_event)
{
    // Never called if NRF_DRV_SPI_FLAG_NO_XFER_EVT_HANDLER flag is used
}

void spi_init()
{
    uint32_t err_code;
    nrf_drv_spi_config_t config = NRF_DRV_SPI_DEFAULT_CONFIG;
    config.mosi_pin = SPI_MOSI_PIN;
    config.miso_pin = SPI_MISO_PIN;
    config.sck_pin = SPI_SCL_PIN;
    config.ss_pin = SPI_CS_PIN;
    err_code = nrf_drv_spi_init(&spi, &config, spi_event_handler);
    APP_ERROR_CHECK(err_code);
    
    // Setting up an SPI transfer using EasyDMA
    nrf_drv_spi_xfer_desc_t xfer = NRF_DRV_SPI_XFER_TRX(&p_tx_data, 1, (uint8_t *)p_rx_data, BYTES_PR_XFER);
    
    uint32_t flags = NRF_DRV_SPI_FLAG_HOLD_XFER           |
                     NRF_DRV_SPI_FLAG_RX_POSTINC          |
                     NRF_DRV_SPI_FLAG_REPEATED_XFER       |
                     NRF_DRV_SPI_FLAG_NO_XFER_EVT_HANDLER;
    
    err_code = nrf_drv_spi_xfer(&spi, &xfer, flags);
    APP_ERROR_CHECK(err_code);
    
    // Get SPI START task address and END event address
    start_spi_task_addr = nrf_drv_spi_start_task_get(&spi);
    spi_end_evt_addr = nrf_drv_spi_end_event_get(&spi);
   

}
    

/**
 * @brief Handler for timer events.
 */
void timer_timeout_event_handler(nrf_timer_event_t event_type, void* p_context)
{
    // Handler must be declared and passed to nrf_drv_timer_init(), 
    // but is not used if nrf_drv_timer_extended_compare() is called with enable_int flag = false
}


/**
 * @brief Handler for timer events.
 */
void counter_cc0_event_handler(nrf_timer_event_t event_type, void* p_context)
{
    // Read out tour buffers here. 
    // Stop timers and hence, stop SPI transfers.
    nrf_drv_spi_uninit(&spi);
    nrf_gpio_pin_set(SPI_CS_PIN); // The SPI driver does not take care of this NRF_DRV_SPI_FLAG_NO_XFER_EVT_HANDLER flag is used with nrf_drv_spi_xfer().
    nrf_drv_timer_disable(&timer_timer);
    nrf_drv_timer_disable(&timer_counter);
}

void timer_init()
{
    uint32_t err_code = NRF_SUCCESS;
    
    /***************
     * Init timer to start SPI transfers on CC0 event
     **************/
    nrf_drv_timer_config_t timer_timout_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    err_code = nrf_drv_timer_init(&timer_timer, &timer_timout_cfg, timer_timeout_event_handler);
    APP_ERROR_CHECK(err_code);

    nrf_drv_timer_extended_compare(&timer_timer, NRF_TIMER_CC_CHANNEL0, 16000, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, false);
    
    timer0_timeout_cc0_evt_addr = nrf_drv_timer_event_address_get(&timer_timer, NRF_TIMER_EVENT_COMPARE0);
    
    
    /***************
     * Init Counter to count SPI transfers
     **************/
    nrf_drv_timer_config_t timer_counter_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    timer_counter_cfg.mode = NRF_TIMER_MODE_COUNTER;
    err_code = nrf_drv_timer_init(&timer_counter, &timer_counter_cfg, counter_cc0_event_handler);
    APP_ERROR_CHECK(err_code);

    nrf_drv_timer_extended_compare(&timer_counter, NRF_TIMER_CC_CHANNEL0, NUMBER_OF_XFERS, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);
    
    counter1_count_task_addr = nrf_drv_timer_task_address_get(&timer_counter, NRF_TIMER_TASK_COUNT);
}


void ppi_init()
{
    uint32_t err_code;
    nrf_ppi_channel_t ppi_ch_timer_cc0_start_spi;
    nrf_ppi_channel_t ppi_ch_spi_end_counter1_count;
    
    /***************
     * Init timer0 timout to start SPI
     **************/
    err_code = nrf_drv_ppi_channel_alloc(&ppi_ch_timer_cc0_start_spi);
    APP_ERROR_CHECK(err_code);
    
    err_code = nrf_drv_ppi_channel_assign(ppi_ch_timer_cc0_start_spi, timer0_timeout_cc0_evt_addr, start_spi_task_addr);
    APP_ERROR_CHECK(err_code);
    
    /***************
     * Init SPI END event causes COUNTER 1 to increment
     **************/
    err_code = nrf_drv_ppi_channel_alloc(&ppi_ch_spi_end_counter1_count);
    APP_ERROR_CHECK(err_code);
    
    err_code = nrf_drv_ppi_channel_assign(ppi_ch_spi_end_counter1_count, spi_end_evt_addr, counter1_count_task_addr);
    APP_ERROR_CHECK(err_code);
    
    // Enable both configured PPI channels
    err_code = nrf_drv_ppi_channel_enable(ppi_ch_timer_cc0_start_spi);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_enable(ppi_ch_spi_end_counter1_count);
    APP_ERROR_CHECK(err_code);
}


/**
 * @brief Function for application main entry.
 */
int main(void)
{
    nrf_gpio_range_cfg_output(LED_1, LED_4);
    
    spi_init();
    timer_init();
    ppi_init();
    
    // Start timers and transfers
    nrf_drv_timer_enable(&timer_timer);
    nrf_drv_timer_enable(&timer_counter);
    
    
    while (true)
    {
        __WFI();
    }
}





///** @} */
